﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form, IMessageForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 로그인 여부
        /// </summary>
        private bool isLogIn = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            FormClosing                           += Form_FormClosing;
            this.loginButton.Click                += loginButton_Click;
            this.logoutButton.Click               += logOutButton_Click;
            this.주식기본정보요청Button.Click     += 주식기본정보요청Button_Click;
            this.주식일봉차트조회요청Button.Click += 주식일봉차트조회요청Button_Click;
            this.일별거래상세요청Button.Click     += 일별거래상세요청Button_Click;
            this.종목코드Button.Click             += 종목코드Button_Click;
            this.hts.OnEventConnect               += hts_OnEventConnect;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 닫을 경우 처리하기 - Form_FormClosing(sender, e)

        /// <summary>
        /// 폼 닫을 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_FormClosing(object sender, FormClosingEventArgs e)
        {
            if(this.isLogIn)
            {
                LogOut();
            }
        }

        #endregion
        #region HTS 이벤트 연결시 처리하기 - hts_OnEventConnect(sender, e)

        /// <summary>
        /// HTS 이벤트 연결시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void hts_OnEventConnect(object sender, AxKHOpenAPILib._DKHOpenAPIEvents_OnEventConnectEvent e)
        {
            if(e.nErrCode == 0) // 성공시
            {
                AddMessage("CONNECT - 로그인 처리 성공");
            }
            else
            {
                AddMessage("CONNECT - 로그인 처리 실패 : {0}", e.nErrCode);
            }
        }

        #endregion
        #region 로그인 버튼 클릭시 처리하기 - loginButton_Click(sender, e)

        /// <summary>
        /// 로그인 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void loginButton_Click(object sender, EventArgs e)
        {
            if(this.hts.CommConnect() == 0)
            {
                AddMessage("로그인 창 열기를 성공했습니다.");

                this.isLogIn = true;
            }
            else
            {
                AddMessage("로그인 창 열기를 실패했습니다.");
            }
        }

        #endregion
        #region 로그아웃 버튼 클릭시 처리하기 - logOutButton_Click(sender, e)

        /// <summary>
        /// 로그아웃 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void logOutButton_Click(object sender, EventArgs e)
        {
            LogOut();

            AddMessage("로그아웃 했습니다.");
        }

        #endregion

        #region 주식기본정보요청 버튼 클릭시 처리하기 - 주식기본정보요청Button_Click(sender, e)

        /// <summary>
        /// 주식기본정보요청 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void 주식기본정보요청Button_Click(object sender, EventArgs e)
        {   
            AddMessage("주식기본정보요청을 시작합니다.");

            using(var request = new 주식기본정보요청(this, this.hts))
            {
                request.InputValueDictionary.Add("종목코드", "105560"); // KB금융지주

                List<주식기본정보항목> resultList = request.GetList();

                if(resultList != null)
                {
                    string json = Newtonsoft.Json.JsonConvert.SerializeObject(resultList[0]);

                    this.resultRichTextBox.Text = json;

                    AddMessage("주식기본정보요청을 종료합니다.");
                }
                else
                {
                    AddMessage("주식기본정보요청을 실패했습니다.");
                }
            }
        }

        #endregion
        #region 주식일봉차트조회요청 버튼 클릭시 처리하기 - 주식일봉차트조회요청Button_Click(sender, e)

        /// <summary>
        /// 주식일봉차트조회요청 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void 주식일봉차트조회요청Button_Click(object sender, EventArgs e)
        {
            AddMessage("주식일봉차트조회요청을 시작합니다.");

            using(var request = new 주식일봉차트조회요청(this, this.hts, TransactionDataType.MultipleFirstPage))
            {
                request.InputValueDictionary.Add("종목코드"    , "105560"  ); // KB금융지주
                request.InputValueDictionary.Add("기준일자"    , "20220204");
                request.InputValueDictionary.Add("수정주가구분", "1"       );

                List<주식일봉차트항목> resultList = request.GetList();

                if(resultList != null)
                {
                    string json = Newtonsoft.Json.JsonConvert.SerializeObject(resultList);

                    this.resultRichTextBox.Text = json;

                    AddMessage("주식일봉차트조회요청을 종료합니다.");
                }
                else
                {
                    AddMessage("주식일봉차트조회요청을 실패했습니다.");
                }
            }
        }

        #endregion
        #region 일별거래상세요청 버튼 클릭시 처리하기 - 일별거래상세요청Button_Click(sender, e)

        /// <summary>
        /// 일별거래상세요청 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void 일별거래상세요청Button_Click(object sender, EventArgs e)
        {
            AddMessage("일별거래상세요청을 시작합니다.");

            using(var request = new 일별거래상세요청(this, this.hts, TransactionDataType.MultipleFirstPage))
            {
                request.InputValueDictionary.Add("종목코드", "105560"  ); // KB금융지주
                request.InputValueDictionary.Add("시작일자", "20220113");

                List<일별거래상세항목> resultList = request.GetList();

                if(resultList != null)
                {
                    string json = Newtonsoft.Json.JsonConvert.SerializeObject(resultList);

                    this.resultRichTextBox.Text = json;

                    AddMessage("일별거래상세요청을 종료합니다.");
                }
                else
                {
                    AddMessage("일별거래상세요청을 실패했습니다.");
                }
            }
        }

        #endregion
        #region 종목코드 버튼 클릭시 처리하기 - 종목코드Button_Click(sender, e)

        /// <summary>
        /// 종목코드 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void 종목코드Button_Click(object sender, EventArgs e)
        {
            string result = this.hts.GetCodeListByMarket("0");

            this.resultRichTextBox.Text = result;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 메시지 지우기 - ClearMessage()

        /// <summary>
        /// 메시지 지우기
        /// </summary>
        private void ClearMessage()
        {
            this.messageListBox.Items.Clear();
        }

        #endregion
        #region 메시지 추가하기 - AddMessage(formatString, argumentArray)

        /// <summary>
        /// 메시지 추가하기
        /// </summary>
        /// <param name="formatString">포맷 문자열</param>
        /// <param name="argumentArray">인자 배열</param>
        public void AddMessage(string formatString, params object[] argumentArray)
        {
            string source = string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), string.Format(formatString, argumentArray));

            Invoke(new MethodInvoker(() => { this.messageListBox.Items.Insert(0, source); }));
        }

        #endregion
        #region 로그 아웃 하기 - LogOut()

        /// <summary>
        /// 로그 아웃 하기
        /// </summary>
        private void LogOut()
        {
            this.hts.DisconnectRealData("0001");

            this.hts.CommTerminate();

            this.isLogIn = false;
        }

        #endregion
    }
}